@@ -11,7 +11,9 @@ import re
 
 from mercurial.i18n import _
 from mercurial import (
+    encoding,
     hook,
+    pycompat,
     util,
 )
 
@@ -136,8 +138,8 @@ def createlog(ui, directory=None, root="", rlog=True, cache=None):
         except IOError:
             raise logerror(_('not a CVS sandbox'))
 
-        if prefix and not prefix.endswith(os.sep):
-            prefix += os.sep
+        if prefix and not prefix.endswith(pycompat.ossep):
+            prefix += pycompat.ossep
 
         # Use the Root file in the sandbox, if it exists
         try:
@@ -146,7 +148,7 @@ def createlog(ui, directory=None, root="", rlog=True, cache=None):
             pass
 
     if not root:
-        root = os.environ.get('CVSROOT', '')
+        root = encoding.environ.get('CVSROOT', '')
 
     # read log cache if one exists
     oldlog = []
